--[[

	InventoryOnPar : An Inventory addon that shows when you have outgrown your equipment
		copyright 2006 by Pericles
	
]]

--[[

v0.01 - Initial release

v0.02 - Added /iop version
		Fixed problem with loading of polearms
		
v0.03 - Fixed problem with loading of thrown weapons

v0.04 - Now shows empty slots in white

v0.10 - Now uses a graphical window to display data

v0.11 - Fixed issue with no Helm giving colour error

v0.20 - Added a button on character inventory screen to open On Par
        Added description field for slots
        Left justified text
        
v0.21 - Tidied up no longer needed local vars
        Added localization
		
v0.22 - Changed Item Level routine to take player level and return colour as well as item level		

v0.30 - Changed Item Level to account for items with multiple levels

V1.00 - Added Item Level to game tooltip - still need to work out "Required Level" text from item tooltip

v1.10 - now works with chat text tooltip and lootlink tooltips

v1.20 - now also works with EquipCompare comparison tooltips

v1.21 - deals with items that contain " of ..." e.g., "of the boar" and looks up the shortened name instead

v1.30 - deals with items that have multiple levels by reading "Required Level" text from item tooltip after 
		v1.21 they were always showing as -3, -5 etc thus always grey.
		
v1.40 - shows item ITEM_RARITY in brackets after item name

v1.41 - fix bug for lootlink

v1.50 - updated for patch 1.10

v1.51 - fix for missing Item Levels on main tooltip post patch 1.10

v2.00 - added new items content from patch 1.10 includes librams, idols, totems & dungeon sets

--]]

local INVENTORYONPAR_VERSION = 2.00;

--------------------------------------------------------------------------------------------------
-- Local variables
--------------------------------------------------------------------------------------------------

colourRed    = "ffff0000";	-- red = 5 lvls above char lvl or better
colourOrange = "ffff8000";	-- orange = 2 to 4 lvls above char lvl
colourYellow = "ffffff00"	-- yellow = 1 lvl below to 1 lvl above char lvl
colourGreen  = "ff1eff00";	-- green = 2 to 4 lvls below char lvl
colourGrey   = "ff9d9d9d";	-- gray = 5 lvls below char lvl or worse
colourBlue   = "ff20d0ff";	-- unknown, light blue
colourWhite  = "ffffffff";	-- slot empty - white

colourDarkBlue = "ff0070dd" -- item ITEM_RARITY dark blue
colourPurple = "ffa335ee"   -- item ITEM_RARITY purple
colourGold   = "ffffd700"   -- item ITEM_RARITY gold

POOR = "Poor";
COMMON = "Common";
UNCOMMON = "Uncommon";
RARE = "Rare";
EPIC = "Epic";
LEGENDARY = "Legendary";
ARTIFACT = "Artifact";

ITEM_RARITY = {};
ITEM_RARITY[0]={ name=POOR, colour=colourGrey };
ITEM_RARITY[1]={ name=COMMON, colour=colourWhite };
ITEM_RARITY[2]={ name=UNCOMMON, colour=colourGreen };
ITEM_RARITY[3]={ name=RARE, colour=colourDarkBlue };
ITEM_RARITY[4]={ name=EPIC, colour=colourPurple };
ITEM_RARITY[5]={ name=LEGENDARY, colour=colourOrange };
ITEM_RARITY[6]={ name=ARTIFACT, colour=colourGold };

--------------------------------------------------------------------------------------------------
-- Create empty main table
--------------------------------------------------------------------------------------------------

ITEMS_DB = {};

--------------------------------------------------------------------------------------------------
-- Event functions
--------------------------------------------------------------------------------------------------

function InventoryOnPar_Initialise()
	-- Register our slash command
	SlashCmdList["INVENTORYONPAR"] = function(msg)
		InventoryOnPar_SlashCommandHandler(msg);
	end
	SLASH_INVENTORYONPAR1 = "/iop";
	
	if(DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage("\nPericles's InventoryOnPar AddOn v"..INVENTORYONPAR_VERSION.." loaded");
	end
	-- hook the item tooltip 
	IOP_GameTooltip_Orig = GameTooltip:GetScript("OnShow");
	GameTooltip:SetScript("OnShow", IOP_GameTooltip_OnShow);
	IOP_ItemRefTooltip_Orig = ItemRefTooltip:GetScript("OnShow");
	ItemRefTooltip:SetScript("OnShow", IOP_ItemRefTooltip_OnShow);
	if (LootLinkTooltip) then
		IOP_LootLinkTooltip_Orig = LootLinkTooltip:GetScript("OnShow");
		LootLinkTooltip:SetScript("OnShow", IOP_LootLinkTooltip_OnShow);
	end
	-- now check if EquipCompare Comparison tooltips loaded
	if (ComparisonTooltip1) then
		IOP_ComparisonTooltip1_Orig = ComparisonTooltip1:GetScript("OnShow");
		ComparisonTooltip1:SetScript("OnShow", IOP_ComparisonTooltip1_OnShow);
	end
	if (ComparisonTooltip2) then
		IOP_ComparisonTooltip2_Orig = ComparisonTooltip2:GetScript("OnShow");
		ComparisonTooltip2:SetScript("OnShow", IOP_ComparisonTooltip2_OnShow);
	end
	InventoryOnParFrame:Hide();
	IOP_LoadClassData();
	-- IOP_Show:setText(BUTTONTEXT); need to get syntax for setting button text to cope with localization
end

--------------------------------------------------------------------------------------------------
-- Handler functions
--------------------------------------------------------------------------------------------------

function InventoryOnPar_SlashCommandHandler(msg)
	if(msg == "version") then
		DEFAULT_CHAT_FRAME:AddMessage("InventoryOnPar Version : "..INVENTORYONPAR_VERSION);
	else
		ShowItemLevels();
	end
end

function ShowItemLevels()
	textstring="|c"..colourWhite.."Inventory Status is : (Player Level : "..UnitLevel("player")..")";
	for index = 1, getn(INVENTORY_SLOT_LIST), 1 do
		INVENTORY_SLOT_LIST[index].id = GetInventorySlotInfo(INVENTORY_SLOT_LIST[index].name);
		local slotLink = GetInventoryItemLink("player", INVENTORY_SLOT_LIST[index].id);
		if (slotLink ~= nil) then
			local _, _, itemCode = strfind(slotLink, "(%d+):");
			local itemName, _, itemRarity, itemMinLevel, _, itemType = GetItemInfo(itemCode);
			local itemLevel, levelColour = IOP_GetItemLevel(itemName, UnitLevel("player"), itemMinLevel);
			textstring=textstring.."\n|c"..levelColour..INVENTORY_SLOT_LIST[index].desc.." - |c"..levelColour.." lvl:"..itemLevel.." - "..itemName;
			textstring=textstring.."  |c"..ITEM_RARITY[itemRarity].colour.."["..ITEM_RARITY[itemRarity].name.."]";
		else
			textstring=textstring.."\n|c"..colourWhite..INVENTORY_SLOT_LIST[index].desc.." - |c"..colourWhite.." Slot Empty";
		end
	end
	InventoryOnParText:SetJustifyH("LEFT");
	InventoryOnParText:SetJustifyV("MIDDLE");
	InventoryOnParText:SetText(textstring);
	InventoryOnParFrame:Show();
end

function IOP_GameTooltip_OnShow()
	-- call original event for GameTooltip:OnShow()
	if IOP_GameTooltip_Orig then
		IOP_GameTooltip_Orig(event)
	end
	
	-- add Item Level info to tooltip if the tooltip has at least 1 line
	local IOP_tooltip = getglobal("GameTooltipTextLeft1");
	if (IOP_tooltip) then
		local reqLevel = IOP_GetReqLevel("GameTooltip");
		local itemLevel, levelColour = IOP_GetItemLevel(IOP_tooltip:GetText(), UnitLevel("player"), reqLevel);
		if (levelColour ~= colourBlue) then
			-- suppress tooltip line if unknown 
			GameTooltip:AddLine("|c"..levelColour.."Item Level : "..itemLevel);
			GameTooltip:Show()
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("|c00bfffff hmm it is null");
	end
end

function IOP_ItemRefTooltip_OnShow()
	-- call original event for ItemRefTooltip:OnShow()
	if IOP_ItemRefTooltip_Orig then
		IOP_ItemRefTooltip_Orig(event)
	end
	
	-- add Item Level info to tooltip if the tooltip has at least 1 line
	local IOP_tooltip = getglobal("ItemRefTooltipTextLeft1");
	if (IOP_tooltip) then
		local reqLevel = IOP_GetReqLevel("ItemRefTooltip");
		local itemLevel, levelColour = IOP_GetItemLevel(IOP_tooltip:GetText(), UnitLevel("player"), reqLevel);
		if (levelColour ~= colourBlue) then
			-- suppress tooltip line if unknown 
			ItemRefTooltip:AddLine("|c"..levelColour.."Item Level : "..itemLevel);
			ItemRefTooltip:Show()
		end
	end
end

function IOP_LootLinkTooltip_OnShow()
	-- call original event for LootLinkTooltip:OnShow()
	if IOP_LootLinkTooltip_Orig then
		IOP_LootLinkTooltip_Orig(event)
	end
	
	-- add Item Level info to tooltip if the tooltip has at least 1 line
	local IOP_tooltip = getglobal("LootLinkTooltipTextLeft1");
	if (IOP_tooltip) then
		local reqLevel = IOP_GetReqLevel("LootLinkTooltip");
		local itemLevel, levelColour = IOP_GetItemLevel(IOP_tooltip:GetText(), UnitLevel("player"), reqLevel);
		if (levelColour ~= colourBlue) then
			-- suppress tooltip line if unknown 
			LootLinkTooltip:AddLine("|c"..levelColour.."Item Level : "..itemLevel);
			LootLinkTooltip:Show()
		end
	end
end

function IOP_ComparisonTooltip1_OnShow()
	-- call original event for ComparisonTooltip1TextLeft1:OnShow()
	if IOP_ComparisonTooltip1_Orig then
		IOP_ComparisonTooltip1_Orig(event)
	end
	
	-- add Item Level info to tooltip if the tooltip has at least 1 line
	local IOP_tooltip = getglobal("ComparisonTooltip1TextLeft1");
	if (IOP_tooltip) then
		local reqLevel = IOP_GetReqLevel("ComparisonTooltip1");
		local itemLevel, levelColour = IOP_GetItemLevel(IOP_tooltip:GetText(), UnitLevel("player"), reqLevel);
		if (levelColour ~= colourBlue) then
			-- suppress tooltip line if unknown 
			ComparisonTooltip1:AddLine("|c"..levelColour.."Item Level : "..itemLevel);
			ComparisonTooltip1:Show()
		end
	end
end

function IOP_ComparisonTooltip2_OnShow()
	-- call original event for ComparisonTooltip1TextLeft2:OnShow()
	if IOP_ComparisonTooltip2_Orig then
		IOP_ComparisonTooltip2_Orig(event)
	end
	
	-- add Item Level info to tooltip if the tooltip has at least 1 line
	local IOP_tooltip = getglobal("ComparisonTooltip2TextLeft1");
	if (IOP_tooltip) then
		local reqLevel = IOP_GetReqLevel("ComparisonTooltip2");
		local itemLevel, levelColour = IOP_GetItemLevel(IOP_tooltip:GetText(), UnitLevel("player"), reqLevel);
		if (levelColour ~= colourBlue) then
			-- suppress tooltip line if unknown 
			ComparisonTooltip2:AddLine("|c"..levelColour.."Item Level : "..itemLevel);
			ComparisonTooltip2:Show()
		end
	end
end

--------------------------------------------------------------------------------------------------
-- Main functions
--------------------------------------------------------------------------------------------------

function IOP_GetReqLevel(fieldname) 
	local iStart, iEnd, value;
	for index = 2, 30 do
		field = getglobal(fieldname.."TextLeft"..index);
		if(field and field:IsVisible()) then
			linetext = field:GetText();
			if (linetext) then
				iStart, iEnd, value = string.find(linetext, "Requires Level (%d+)");
				if (value) then
					return value;
				end
			end
		end
	end
	return 0;
end

function IOP_LoadClassData() 
	local class = UnitClass("player")

-- first load data all classes can use
	IOP_LoadFishingPoleData();
	IOP_LoadMiscData();
	IOP_LoadMiscArmourData();
	IOP_LoadClothArmourData();
	IOP_LoadTotemData();
	IOP_LoadIdolData();
	IOP_LoadLibramData();

-- now load class specific data
	if (class == "Druid") then 
		IOP_LoadDaggerData();
		IOP_LoadStaffData();
		IOP_LoadFistData(); 
		IOP_LoadPoleArmData();
		IOP_LoadOneHandedMaceData(); 

		IOP_LoadLeatherArmourData();
	end;
	if (class == "Hunter") then 
		IOP_LoadOneHandedAxeData();
		IOP_LoadDaggerData();
		IOP_LoadThrownData();
		IOP_LoadFistData(); 
		IOP_LoadCrossbowData();
		IOP_LoadGunData();
		IOP_LoadBowData();
		IOP_LoadPoleArmData();
		IOP_LoadStaffData();
		IOP_LoadOneHandedSwordData();
		IOP_LoadTwoHandedSwordData();
		IOP_LoadTwoHandedAxeData();

		IOP_LoadLeatherArmourData();
		IOP_LoadMailArmourData();
	end;
	if (class == "Mage") then 
		IOP_LoadStaffData();
		IOP_LoadWandData(); 
		IOP_LoadDaggerData();
		IOP_LoadOneHandedSwordData();
	end;
	if (class == "Paladin") then 
		IOP_LoadPoleArmData();
		IOP_LoadOneHandedAxeData();
		IOP_LoadOneHandedMaceData();
		IOP_LoadOneHandedSwordData();
		IOP_LoadTwoHandedMaceData();
		IOP_LoadTwoHandedSwordData();
		IOP_LoadTwoHandedAxeData();

		IOP_LoadLeatherArmourData();
		IOP_LoadMailArmourData();
		IOP_LoadPlateArmourData();
		IOP_LoadShieldData();
	end;
	if (class == "Priest") then 
		IOP_LoadWandData(); 
		IOP_LoadStaffData();
		IOP_LoadOneHandedMaceData(); 
		IOP_LoadDaggerData();
	end;
	if (class == "Rogue") then 
		IOP_LoadDaggerData();
		IOP_LoadThrownData();
		IOP_LoadFistData(); 
		IOP_LoadCrossbowData();
		IOP_LoadGunData();
		IOP_LoadBowData();
		IOP_LoadOneHandedMaceData();
		IOP_LoadOneHandedSwordData();

		IOP_LoadLeatherArmourData();
	end;
	if (class == "Shaman") then 
		IOP_LoadOneHandedAxeData();
		IOP_LoadOneHandedMaceData();
		IOP_LoadStaffData();
		IOP_LoadFistData(); 
		IOP_LoadDaggerData();
		
		IOP_LoadLeatherArmourData();
		IOP_LoadMailArmourData();
		IOP_LoadShieldData();
	end;
	if (class == "Warlock") then 
		IOP_LoadWandData(); 
		IOP_LoadStaffData();
		IOP_LoadOneHandedSwordData(); 
		IOP_LoadDaggerData();
	end;
	if (class == "Warrior") then 
		IOP_LoadDaggerData();
		IOP_LoadOneHandedAxeData();
		IOP_LoadOneHandedMaceData();
		IOP_LoadOneHandedSwordData();
		IOP_LoadFistData(); 
		IOP_LoadCrossbowData();
		IOP_LoadGunData();
		IOP_LoadBowData();
		IOP_LoadPoleArmData();
		IOP_LoadStaffData();
		IOP_LoadThrownData();
		IOP_LoadTwoHandedAxeData();
		IOP_LoadTwoHandedMaceData();
		IOP_LoadTwoHandedSwordData();
		
		IOP_LoadLeatherArmourData();
		IOP_LoadMailArmourData();
		IOP_LoadPlateArmourData();
		IOP_LoadShieldData();
	end;
	
	DEFAULT_CHAT_FRAME:AddMessage("|c00bfffffPericles's InventoryOnPar "..class.." Data Loaded");
end 

function IOP_GetItemLevel(itemName, playerLevel, itemMinLevel)
	if (itemName) then
		local item = ITEMS_DB[itemName];
		local first, last = string.find(itemName, " of ");
		if (not item and first) then
			local shortName = string.sub(itemName, 1, first - 1);
			item = ITEMS_DB[shortName];
		end
		if (item) then
			local levelDiff = 0;
			local colour = colourWhite;
			local itemLevel = 0;
			if (string.sub(item.level,1,1) == "-") then
				-- we have an item with multiple levels eg: 23,33,43,53,63 stored as -3 in level database.
				-- so use minLevel - item.Level to get actual item level. eg: lvl 33 is minlvl 30 - (-3) = 33.
				itemLevel = itemMinLevel - item.level;
			else
				itemLevel = item.level;
			end
			if (itemLevel == 0) then
				return 0, colourWhite;
			end
			levelDiff = itemLevel - playerLevel;
			if (levelDiff >= 5) then
				colour = colourRed;
			end
			if (levelDiff <= 4 and levelDiff >=2) then
				colour = colourOrange;
			end
			if (levelDiff <= 1 and levelDiff >=-1) then
				colour = colourYellow;
			end
			if (levelDiff <= -2 and levelDiff >=-4) then
				colour = colourGreen;
			end
			if (levelDiff <= -5) then
				colour = colourGrey;
			end
			return itemLevel, colour;
		else
			--DEFAULT_CHAT_FRAME:AddMessage("|c00bfffff no such item");
			return "not found", colourBlue;
		end
	else 
		--DEFAULT_CHAT_FRAME:AddMessage("|c00bfffffin getItemLevel itemCode is nil");
		return "not found", colourBlue;
	end
	
end